# -*- coding: utf-8 -*-
#
# django-agpl -- tools to aid releasing Django projects under the AGPL
# Copyright (C) 2008, 2009 Chris Lamb <chris@chris-lamb.co.uk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import django_agpl

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

__all__ = ('get_file_list', 'create_tarball', 'get_fileobj')

def create_tarball(mode='w'):
    import tarfile
    fileobj = get_fileobj()
    f = tarfile.open('download', mode, fileobj)
    for location, name in get_file_list():
        f.add(location, name)
    f.close()
    return fileobj

def get_fileobj():
    return StringIO()

def matches_any(name, candidates):
    for pat in candidates:
        if re.search(pat, name):
            return True
    return False

def get_file_list():
    try:
        app_root = settings.AGPL_ROOT
    except AttributeError:
        raise ImproperlyConfigured(
            "You must specify 'AGPL_ROOT' in your project's settings"
        )

    try:
        prefix = settings.AGPL_PREFIX
    except AttributeError:
        # No prefix
        prefix = ''

    try:
        MY_EXCLUDE_FILES = settings.AGPL_EXCLUDE_FILES
    except AttributeError:
        # Fallback to default exclude list for files
        MY_EXCLUDE_FILES = django_agpl.EXCLUDE_FILES

    try:
        MY_EXCLUDE_DIRS = settings.AGPL_EXCLUDE_DIRS
    except AttributeError:
        # Fallback to default exclude list for directories
        MY_EXCLUDE_DIRS = django_agpl.EXCLUDE_DIRS

    for root, dirs, files in os.walk(app_root):
        for filename in files:
            if matches_any(filename, MY_EXCLUDE_FILES):
                continue

            yield (
                os.path.join(root, filename),
                os.path.join(
                    prefix,
                    root[len(app_root) + 1:],
                    filename,
                ),
            )

        for subdir in dirs:
            if matches_any(subdir, MY_EXCLUDE_DIRS):
                try:
                    dirs.remove(subdir)
                except ValueError:
                    pass



# -*- coding: utf-8 -*-
#
# django-agpl -- tools to aid releasing Django projects under the AGPL
# Copyright (C) 2008, 2009 Chris Lamb <chris@chris-lamb.co.uk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.http import HttpResponse
from django.core.exceptions import ImproperlyConfigured

from django_agpl.signals import project_downloaded

__all__ = ('DownloadHTTPResponse',)

class DownloadHttpResponse(HttpResponse):
    def __init__(self, fileobj, *args, **kwargs):
        extension = kwargs.pop('extension', '')
        super(DownloadHttpResponse, self).__init__(
            fileobj.getvalue(),
            *args,
            **kwargs
        )

        try:
            prefix = settings.AGPL_FILENAME_PREFIX
        except AttributeError:
            raise ImproperlyConfigured(
                "You must specify 'AGPL_FILENAME_PREFIX' in your "
                "project's settings"
            )

        self['Content-disposition'] = 'attachment; filename=%s%s' % \
            (prefix, extension)
        self.write(fileobj.getvalue())

        project_downloaded.send(sender=None)



# -*- coding: utf-8 -*-
#
# django-agpl -- tools to aid releasing Django projects under the AGPL
# Copyright (C) 2008, 2009 Chris Lamb <chris@chris-lamb.co.uk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls.defaults import *

urlpatterns = patterns('django_agpl.views',
   url(r'^tar$', 'tar', name='download-tar'),
   url(r'^zip$', 'zip', name='download-zip'),
   url(r'^targz$', 'targz', name='download-targz'),
   url(r'^tarbz2$', 'tarbz2', name='download-tarbz2'),
)



# -*- coding: utf-8 -*-
#
# django-agpl -- tools to aid releasing Django projects under the AGPL
# Copyright (C) 2008, 2009 Chris Lamb <chris@chris-lamb.co.uk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings

from django_agpl.http import DownloadHttpResponse
from django_agpl.utils import get_file_list, create_tarball, get_fileobj

__all__ = ('tar', 'tarbz', 'tarbz2', 'zip')

def tar(self):
    return DownloadHttpResponse(
        create_tarball(),
        extension='.tar',
        mimetype='application/x-gtar',
    )

def targz(self):
    return DownloadHttpResponse(
        create_tarball(mode='w|gz'),
        extension='.tar.gz',
        mimetype='application/x-gtar',
    )

def tarbz2(self):
    return DownloadHttpResponse(
        create_tarball(mode='w|bz2'),
        extension='.tar.bz2',
        mimetype='application/x-gtar',
    )

def zip(self):
    import zipfile

    fileobj = get_fileobj()
    f = zipfile.ZipFile(fileobj, mode='w')
    for location, name in get_file_list():
        f.write(location, name)
    f.close()

    return DownloadHttpResponse(
        fileobj,
        extension='.zip',
        mimetype='application/x-gtar',
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# django-agpl -- tools to aid releasing Django projects under the AGPL
# Copyright (C) 2008, 2009 Chris Lamb <chris@chris-lamb.co.uk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import glob

def get_files(dirs, prefix=None):
    oldpwd = os.path.abspath(os.curdir)
    if prefix:
        os.chdir(prefix)

    result = []
    for dir in dirs:
        for dirpath, dirnames, filenames in os.walk(dir):
            for filename in filenames:
                result.append(os.path.join(dirpath, filename))

    os.chdir(oldpwd)
    return result

setup_args = dict(
    name='django-agpl',
    packages=[
        'django_agpl',
    ],
    author='Chris Lamb',
    author_email='chris@chris-lamb.co.uk',
    package_data={
        'django_agpl': get_files(['templates'], 'django_agpl'),
    },
    data_files = (
        ('share/django_agpl', glob.glob('images/*')),
    ),
)

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(**setup_args)



# -*- coding: utf-8 -*-
#
# django-agpl -- tools to aid releasing Django projects under the AGPL
# Copyright (C) 2008, 2009 Chris Lamb <chris@chris-lamb.co.uk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

EXCLUDE_FILES = [
    r'^site_settings.py$',
    r'^local_settings.py$',
    r'^\.gitignore$',
    r'\.pyc$',
    r'\.sql$',
    r'\.sqlite$',
]

EXCLUDE_DIRS = [
    r'^CVS$',
    r'^site_media$',
    r'^uploads$',
    r'\.git$',
    r'\.svn$',
]
